"""Gravitrax Example Script: Handling Notifications
This Script connects to a Bridge and displays inforation about 
received Notifications as well as the count of all notifications
received at runtime. All protocol fields of the received signal 
can be accessed as Keyword Arguments. The program closes after
10 Notifications are received.
"""

import asyncio
from gravitraxconnect import gravitrax_bridge as gb
from gravitraxconnect import gravitrax_constants
from tkinter import *

#fenster = Tk()
#show = Label(fenster,text="Hallo")
#show.pack()

tiptube_counter = 0
scoop_counter = 0
carousel = 0
start = 0
counter = 0

finished = asyncio.Event()
disconnected = asyncio.Event()

def disconnect_callback(bridge: gb.Bridge, **kwargs):
    """Callback function for disconnects"""
    disconnected.set()

async def notification_callback(bridge: gb.Bridge, **signal):
    """Callback function for incoming Signals
    
    This callback is executed when a Notification i.e a Signal is received.
    After 10 Signals the finished event is set which closes the program.
    """

   
    global tiptube_counter
    global scoop_counter
    global carousel
    global start
    global counter
    def lookup(key, table, prefix):
        try:
            return table[key]
        except (IndexError, KeyError):
            if prefix:
                return f"{prefix}{key}"
            return key

    start = start +1
    # Received Data doesn't fits the communication protocol
    if not (data := signal.get("Header")):
        gb.log_print(f"Received Notification: {data}", bridge=bridge)
    # Received Data does fits the communication protocol
    else:
        # Access relevant Signal information
        # Other kwargs that can be accessed are:
        # Header, Reserved, id and Checksum
        stone = lookup(signal.get("Stone"), gravitrax_constants.DICT_VAL_STONE, None)
        color = lookup(signal.get("Color"), gravitrax_constants.LOOKUP_COLOR, None)
        #id = lookup(signal.get("ID"), gravitrax_constants.ID, None)
        status = lookup(
            signal.get("Status"), gravitrax_constants.DICT_VAL_STATUS, "Color"
        )

        #Hauptteil: Signale auswerten
        #Bahn mit 7 Kugeln starten
        if start == 1:
            for zahl in range(8):
                await asyncio.sleep(1)
                await bridge.send_signal(
                gravitrax_constants.STATUS_STARTER,
                gravitrax_constants.COLOR_RED,
                resends=12,
                )
        
        #TRIGGER 1
        #Trigger 1 zählt lediglich die Kugeln im TipTube und zeigt die Anzahl an

        
        if stone == "finish" and color == "green":
            tiptube_counter = tiptube_counter + 1
            print("In TipTube: ",  tiptube_counter)
            
            if tiptube_counter == 3:
                tiptube_counter = 0
        
        
        #Trigger 2
        #Trigger 2 registriert nur, ob eine Kugel oben im Carousel liegt.
        #Das Carousel hat Priorität, darum wird dann die Switch umgestellt, damit das Carousel ausgelöst wird.
        #Die Variable wird anschließend wieder auf 0 gesetzt

        if stone == "trigger" and color == "green":
                carousel = carousel + 1
                if carousel >= 1:
                    await bridge.send_signal(
                    gravitrax_constants.STATUS_SWITCH,
                    gravitrax_constants.COLOR_BLUE,
                    resends=12,
                    )
            
        #Trigger 3
        #Trigger 3 hat lediglich die Aufgabe, die Switch wieder in Richtung TipTube zu stellen
        if stone == "finish" and color == "blue":
            await bridge.send_signal(
            gravitrax_constants.STATUS_SWITCH,
            gravitrax_constants.COLOR_BLUE,
            resends=12,
            )
            carousel = 0

        #Trigger 4
        #Trigger 4 zählt die Kugeln, die in den Scoop laufen. Sind drei reingelaufen, wird der Scoop dreimal ausgelöst.
        #Anschließend wird die Variable wieder auf 0 gesetzt.
        if stone == "trigger" and color == "red":
            scoop_counter = scoop_counter + 1
            print("Sccop-Counter: ", scoop_counter)
            if scoop_counter == 3:
                for zahl in range(3):
                    await bridge.send_signal(
                    gravitrax_constants.STATUS_SCOOP,
                    gravitrax_constants.COLOR_RED,
                    resends=12,
                    )
                    await asyncio.sleep(3)
                    scoop_counter = 0


        """gb.log_print(
            f"{color} detected from Stone {stone} with Status {status} and id {id}",
            f"( {counter} Notifications received )",
            bridge=bridge,
        )"""
        
    if counter >= 500:
        finished.set()






async def main():
    """Recieve Notifications from the Bridge"""
    bridge = gb.Bridge()
    gb.logger.disabled = False
    gb.log_set_level("INFO")
    gb.log_print("Searching for Bridge")
    try:
        if await bridge.connect(dc_callback=disconnect_callback):
            await bridge.start_bridge_mode()
            await bridge.notification_enable(notification_callback)
            await finished.wait()
            await bridge.notification_disable()
            gb.log_print(f"Received {counter} Notifications", bridge=bridge)
            await bridge.disconnect()
            await disconnected.wait()
    except asyncio.CancelledError:
        return
if __name__ == "__main__":
    asyncio.run(main())
    gb.log_print("Program finished")

#show.mainloop()