"""Gravitrax Example Script: Handling Notifications
This Script connects to a Bridge and displays inforation about 
received Notifications as well as the count of all notifications
received at runtime. All protocol fields of the received signal 
can be accessed as Keyword Arguments. The program closes after
10 Notifications are received.
"""

import asyncio
from gravitraxconnect import gravitrax_bridge as gb
from gravitraxconnect import gravitrax_constants
from tkinter import *

#fenster = Tk()
#show = Label(fenster,text="Hallo")
#show.pack()

durchlauf = 0
counter = 0

finished = asyncio.Event()
disconnected = asyncio.Event()

def disconnect_callback(bridge: gb.Bridge, **kwargs):
    """Callback function for disconnects"""
    disconnected.set()

async def notification_callback(bridge: gb.Bridge, **signal):
    """Callback function for incoming Signals
    
    This callback is executed when a Notification i.e a Signal is received.
    After 10 Signals the finished event is set which closes the program.
    """

   
    global durchlauf
    global start
    global counter
    def lookup(key, table, prefix):
        try:
            return table[key]
        except (IndexError, KeyError):
            if prefix:
                return f"{prefix}{key}"
            return key

    
    # Received Data doesn't fits the communication protocol
    if not (data := signal.get("Header")):
        gb.log_print(f"Received Notification: {data}", bridge=bridge)
    # Received Data does fits the communication protocol
    else:
        # Access relevant Signal information
        # Other kwargs that can be accessed are:
        # Header, Reserved, id and Checksum
        stone = lookup(signal.get("Stone"), gravitrax_constants.DICT_VAL_STONE, None)
        color = lookup(signal.get("Color"), gravitrax_constants.LOOKUP_COLOR, None)
        #id = lookup(signal.get("ID"), gravitrax_constants.ID, None)
        status = lookup(
            signal.get("Status"), gravitrax_constants.DICT_VAL_STATUS, "Color"
        )

        #Hauptteil: Signale auswerten
        #test
        #Finish Trigger 

        if stone == "finish" and color == "green":
            durchlauf = durchlauf + 1
            print("Durchlauf Nummero: ", durchlauf)
            if durchlauf == 2:
                await bridge.send_signal(
                gravitrax_constants.STATUS_SWITCH,
                gravitrax_constants.COLOR_RED,
                resends=12,
                )

        if stone == "trigger" and color == "green":
            print("Der normale Trigger wurde getriggert - macht aber nix!")
        


        """gb.log_print(
            f"{color} detected from Stone {stone} with Status {status} and id {id}",
            f"( {counter} Notifications received )",
            bridge=bridge,
        )"""
        
    if counter >= 50:
        finished.set()






async def main():    
    """Recieve Notifications from the Bridge"""
    bridge = gb.Bridge()
    gb.logger.disabled = False
    gb.log_set_level("INFO")
    gb.log_print("Searching for Bridge")
    try:
        if await bridge.connect(dc_callback=disconnect_callback):
            await bridge.start_bridge_mode()
            await bridge.notification_enable(notification_callback)
            await finished.wait()
            await bridge.notification_disable()
            gb.log_print(f"Received {counter} Notifications", bridge=bridge)
            await bridge.disconnect()
            await disconnected.wait()
    except asyncio.CancelledError:
        return
                
if __name__ == "__main__":
    asyncio.run(main())
    gb.log_print("Program finished")